/*
 * Copyright (C) 2003-2007 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */
package org.exoplatform.services.jcr.storage.value;

import java.io.IOException;
import java.util.List;
import java.util.Properties;

import javax.naming.NamingException;

import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.storage.WorkspaceStorageConnection;

/**
 * Created by The eXo Platform SAS.
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: ValueStoragePlugin.java 35195 2009-08-07 14:48:51Z pnedonosko $
 */

public abstract class ValueStoragePlugin {

  protected List<ValuePluginFilter> filters;

  protected String                  id;

  /**
   * Initialize this plugin.
   * 
   * @param props
   * @throws RepositoryConfigurationException
   * @throws IOException
   * @throws NamingException
   */
  public abstract void init(Properties props) throws RepositoryConfigurationException, IOException;

  public abstract ValueIOChannel openIOChannel() throws IOException;

  /**
   * @return filters
   */
  public final List<ValuePluginFilter> getFilters() {
    return filters;
  }

  public final void setFilters(List<ValuePluginFilter> filters) {
    this.filters = filters;
  }

  public final String getId() {
    return id;
  }

  public final void setId(String id) {
    this.id = id;
  }

  /**
   * Run consistency check operation
   * 
   * @param dataConnection
   *          - connection to metadata storage
   */
  public abstract void checkConsistency(WorkspaceStorageConnection dataConnection);

  public abstract boolean match(String valueDataDescriptor);

}
